#include "../../internal-include/BootLoader/boot_loader.hpp"
#include "../../internal-include/BootLoader/color_debug.hpp"
#include "../../internal-include/GPU/gpu_internal.hpp"
#include <PSX/System/IOPorts/dma_io.hpp>
#include <stdio.h>

extern "C" uint32_t __heap_start;
extern "C" uint32_t __bss_start;
extern "C" uint32_t __bss_end;
extern "C" uint32_t __bss_len;
extern "C" uint32_t __planschi_start;
extern "C" uint32_t __planschi_end;

namespace JabyEngine {
    namespace boot {
        namespace Start {
            static void enable_DMA() {
                DMA_IO::DPCR.write(DMA_IO::DPCR.read().set2(DMA_IO::DPCR::SPUEnable, DMA_IO::DPCR::GPUEnable, DMA_IO::DPCR::CDROMEnable));
            }

            static void setup() {
                __debug_boot_color1(::JabyEngine::GPU::Color24::White());
                enable_DMA();

                __debug_boot_color1(::JabyEngine::GPU::Color24::Red());
                SPU::stop_voices();
                
                __debug_boot_color1(::JabyEngine::GPU::Color24::Green());
                CD::setup();
                __debug_boot_color1(::JabyEngine::GPU::Color24::Blue());
                Timer::setup();

                __debug_boot_color1(::JabyEngine::GPU::Color24::Yellow());
                GPU::setup();
                GPU::display_logo();
                
                //Pause??
                SPU::setup();
            }
        }
    }
    void start() {
        __debug_boot_print0(GPU::Color24::White(), "Starting Planschbecken\n");
        printf("Heap starts @0x%p\n", &__heap_start);
        printf("BSS      from 0x%p to 0x%p (%u)\n", &__bss_start, &__bss_end, __bss_len);
        __debug_boot_print0(GPU::Color24::Green(), "PLANSCHI from 0x%p to 0x%p\n", &__planschi_start, &__planschi_end);
        boot::Start::setup();
        
        __debug_boot_print0(GPU::Color24::Red(), "Running main...\n");
        GPU::internal::wait_vsync(0);
        run();
    }
}